package net.jieyuu.mq;

import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import net.jieyuu.model.CouponRecordMessage;
import net.jieyuu.service.CouponRecordService;
import org.redisson.api.RedissonClient;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.concurrent.locks.Lock;

@Slf4j
@Component
@RabbitListener(queues = "${mqconfig.coupon_release_queue}")
public class CouponMQListener {

    @Autowired
    private CouponRecordService couponRecordService;
    @Autowired
    private RedissonClient redissonClient;

    /**
     * 重复消费幂等性
     * 消费失败，重新入队最大重试次数
     * 消费失败不重新入队，而是通过日志，后插入数据库
     *
     * @param recordMessage
     * @param message
     * @param channel
     * @throws IOException
     */
    @RabbitHandler
    public void releaseCouponRecord(CouponRecordMessage recordMessage, Message message, Channel channel) throws IOException {

        log.info("监听到消息:releaseCouponRecord:{}", recordMessage);
        long msgTag = message.getMessageProperties().getDeliveryTag();
        boolean flag = couponRecordService.releaseCouponRecord(recordMessage);
        // 防止同时解锁多任务并发进入
        // 串行消费不需要加锁
//        Lock lock = redissonClient.getLock("lock:coupon_record_release:" + recordMessage.getTaskId());
//        lock.lock();
        try {
            if (flag) {
                // 确认消息消费成功 不需要重新入队
                channel.basicAck(msgTag, false);
            } else {
                log.error("释放优惠券失败 flag=false,{}", recordMessage.toString());
                // 选择重新入队
                channel.basicReject(msgTag, true);
            }
        } catch (IOException e) {
            log.error("释放优惠券异常:{},msg:{}", e, recordMessage.toString());
            // 选择重新入队
            // 再发生问题就往外抛
            channel.basicReject(msgTag, true);
        }
//        finally {
//            lock.unlock();
//        }
    }


//    @RabbitHandler
//    public void releaseCouponRecord2(String msg, Message message, Channel channel) throws IOException {
//        log.info(msg);
//        channel.basicAck(message.getMessageProperties().getDeliveryTag(), true);
//
//    }
}
