package net.jieyuu.controller;


import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import net.jieyuu.request.LockProductRequest;
import net.jieyuu.service.ProductService;
import net.jieyuu.utils.JsonData;
import net.jieyuu.vo.ProductVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author jieyuu
 * @since 2024-08-07
 */
@Api(tags = "商品模块")
@RestController
@RequestMapping("/api/product/v1")
public class ProductController {
    @Autowired
    private ProductService productService;


    /**
     * 分页查询商品列表
     *
     * @param page
     * @param size
     * @return
     */
    @ApiOperation(value = "分页查询商品列表")
    @GetMapping("page")
    public JsonData page(@ApiParam("当前页")
                         @RequestParam(value = "page", defaultValue = "1") int page,
                         @ApiParam("显示多少条")
                         @RequestParam(value = "size", defaultValue = "20") int size) {
        Map<String, Object> pageResult = productService.page(page, size);
        return JsonData.buildSuccess(pageResult);
    }

    /**
     * 根据id查找商品详情
     *
     * @param productId
     * @return
     */
    @ApiOperation("根据id查询商品详情")
    @GetMapping("detail/{product_id}")
    public JsonData detail(@ApiParam(value = "商品id", required = true) @PathVariable("product_id") long productId) {
        ProductVO productVO = productService.findDetailById(productId);

        return JsonData.buildSuccess(productVO);
    }

    /**
     * 商品库存锁定
     *
     * @return
     */
    @ApiOperation("商品库存锁定")
    @PostMapping("lock_products")
    public JsonData lockProduct(@ApiParam("商品库存锁定") @RequestBody LockProductRequest lockProductRequest) {
        JsonData jsonData = productService.lockProductStock(lockProductRequest);
        return jsonData;
    }


}

